//
//  AppController.m
//  QVis
//
//  Created by Edward Patel on 2008-12-03.
//

#import "AppController.h"

#undef MIN
#undef MAX

#include <MxGL.h>
#include <stdmix.h>
#include <mixio.h>
#include <qslim.h>
#include <MxQMetric3.h>
#include <MxCamera.h>
#include <gfx/arcball.h>
#include <MxPropSlim.h>

extern uint orig_vert_count, orig_face_count;
extern bool isMovingMouse;
extern MxBounds bounds;
extern Arcball arcball;
extern bool will_draw_edges;
extern MxStdModel *orginalModel;

void decimate_callback(const MxPairContraction& conx, float cost);

void slim_to_target(uint target);
void close_output();

void open_file(NSString *filename) 
{
	slim_cleanup();
	
	if (orginalModel) {
		delete orginalModel;
		orginalModel=NULL;
	}
	
	smf = new MxSMFReader;
	
	char *argv[] = {
		"qslim",
	};
	
	process_cmdline(1, argv);
	
	input_file([filename cStringUsingEncoding:NSUTF8StringEncoding]);
	
	if (m) {
		orginalModel = m->clone();

		orig_vert_count = m->vert_count();
		orig_face_count = m->face_count();
		
		if( m->normal_binding() == MX_UNBOUND )
		{
			m->normal_binding(MX_PERFACE);
			m->synthesize_normals();
		}
		
		slim_init();
		
		eslim->contraction_callback = decimate_callback;
		
		bounds.reset();
		for(MxVertexID v=0; v<m->vert_count(); v++)
			bounds.add_point(m->vertex(v));
		bounds.complete();
		
		isMovingMouse = false;
		arcball.bounding_sphere(bounds.center, bounds.radius);		
	}
}

void save_file(NSString *filename) 
{
	if (output_filename) 
		free(output_filename);
	output_filename = strdup([filename cStringUsingEncoding:NSUTF8StringEncoding]);
	if (orginalModel) {
#if 1
		int numfaces = slim->valid_faces;
		MxStdModel *tm, *mm = orginalModel->clone();
		MxPropSlim slim(mm);
		slim.initialize();
		slim.decimate(numfaces);
		tm = m;
		m = mm;
		output_final_model();
		m = tm;
		delete mm;
		close_output();
#else
		// Use original model and decimate to target no faces to save
		int numfaces = slim->valid_faces;
		slim_cleanup();
		m = orginalModel->clone();
		slim_init();
		eslim->contraction_callback = decimate_callback;
		slim->decimate(numfaces);
		output_final_model();
		
		// Use original model again and make normals and decimate to target no faces for viewing again 
		slim_cleanup();
		m = orginalModel->clone();
		if( m->normal_binding() == MX_UNBOUND )
		{
			m->normal_binding(MX_PERFACE);
			m->synthesize_normals();
		}
		slim_init();
		eslim->contraction_callback = decimate_callback;
		slim_to_target(numfaces);
#endif
	}
}

@implementation AppController

- (void)awakeFromNib
{
	[self updateStatus];
}

- (void)updateStatus
{
	int a=0, b=0;
	if (m) {
		a = slim?slim->valid_verts:m->vert_count();
		b = slim?slim->valid_faces:m->face_count();
	}
	[textField setStringValue:[NSString stringWithFormat:@"Original: %uv/%uf   Current: %uv/%uf", 
							   orig_vert_count, 
							   orig_face_count,
							   a,
							   b]];
}

- (void)updateSlider:(id)sender
{
	if (!m) return;
    slim_to_target([sender intValue]);
	[self updateStatus];
	[myView setNeedsDisplay:YES];
}

- (IBAction)updateWireframe:(id)sender
{
	will_draw_edges = [sender integerValue];
	if (!m) return;
	[myView setNeedsDisplay:YES];
}

- (void)openPanelDidEnd:(NSOpenPanel *)panel 
			 returnCode:(int)returnCode  
			contextInfo:(void*)contextInfo
{
	if (returnCode == NSOKButton) {
		
		open_file([panel filename]);
		if (m) {
			[self setTargetMax:m->face_count()];
			[self updateStatus];
		}
		[myView setNeedsDisplay:YES];
	}
}

- (IBAction)open:(id)sender
{
	NSOpenPanel* op = [NSOpenPanel openPanel];
	
	[op setCanChooseDirectories:NO];
	[op setCanChooseFiles:YES];
	[op setAllowsMultipleSelection:NO];
	
	[op setTitle:@"Open model"];
	[op setPrompt:@"Open"];

	[op beginSheetForDirectory:nil
						  file:nil 
						 types:[NSArray arrayWithObjects:@"smf", @"obj", nil]
				modalForWindow:window
				 modalDelegate:self 
				didEndSelector:@selector(openPanelDidEnd:returnCode:contextInfo:) 
				   contextInfo:nil];
}

- (void)savePanelDidEnd:(NSSavePanel *)panel 
			 returnCode:(int)returnCode  
			contextInfo:(void*)contextInfo
{
	if (returnCode == NSOKButton) {
		save_file([panel filename]);
	}
}

- (IBAction)save:(id)sender
{
	NSSavePanel* op = [NSSavePanel savePanel];
	
	[op setCanCreateDirectories:YES];
	
	[op setTitle:@"Save model"];
	[op setPrompt:@"Save"];
	
	[op beginSheetForDirectory:nil
						file:nil 
				modalForWindow:window 
				 modalDelegate:self 
				didEndSelector:@selector(savePanelDidEnd:returnCode:contextInfo:) 
				   contextInfo:nil];
}

- (void)setTargetMax:(int)max
{
	[slider setMaxValue:max];
	[slider setIntegerValue:max];
	[slider setNeedsDisplay];
}

- (void)windowWillClose:(NSNotification *)notification
{
	[self stop:self];
}

@end
