//
//  MyOpenGLView.m
//  QVis
//
//  Created by Edward Patel on 2008-12-03.
//

#import "MyOpenGLView.h"
#import "AppController.h"

#undef MIN
#undef MAX

#include <MxGL.h>
#include <stdmix.h>
#include <mixio.h>
#include <qslim.h>
#include <MxQMetric3.h>
#include <MxCamera.h>
#include <gfx/arcball.h>

uint orig_vert_count, orig_face_count;
uint selected_vertex, selected_face;
bool will_draw_quadrics;
bool will_draw_edges;
bool will_colorize_regions;
bool will_light_scene;
bool isMovingMouse;
double quadric_radius;
MxCamera camera;
MxBounds bounds;
Arcball arcball;
double mesh_color[3];
static int last[2];
void open_file(NSString *filename);
MxStdModel *orginalModel = NULL;

void decimate_callback(const MxPairContraction& conx, float cost)
{
    slim_history_callback(conx, cost);
}

void slim_undo()
{
    MxPairContraction& conx = history->drop();
	
    eslim->apply_expansion(conx);
}

void slim_to_target(uint target)
{
    if(target > m->face_count())  target = m->face_count();
	
    if( target < slim->valid_faces )
        slim->decimate(target);
    else if( target > slim->valid_faces && history )
    {
        while( slim->valid_faces < target && history->length() > 0 )
            slim_undo();
    }
}

static
void draw_quadric(uint v)
{
    mx_draw_quadric(((MxQSlim*)slim)->vertex_quadric(v),
                    quadric_radius, m->vertex(v));
}

@implementation MyOpenGLView

- (id)initWithCoder:(NSCoder *)coder
{
    if (self = [super initWithCoder:coder]) {

    }
    return self;
}

- (void)awakeFromNib
{
    will_record_history = true;
	will_draw_quadrics = false;
	will_draw_edges = false;
	will_colorize_regions = false;
	will_light_scene = true;
	quadric_radius = 0.75;
	mesh_color[0] = mesh_color[1] = mesh_color[2] = 0.05f;
	
	smf = new MxSMFReader;
	
	char *argv[] = {
		"qslim",
	};
	
	process_cmdline(1, argv);
	
	slim_init();

	//output_final_model();
	
	//update_status();
}

- (void)drawRect:(NSRect)rect 
{
	glEnable(GL_DEPTH_TEST);
    glEnable(GL_LIGHTING);
    glEnable(GL_NORMALIZE);
    glShadeModel(GL_SMOOTH);
	
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glMatrixMode(GL_MODELVIEW);
	
    GLfloat mat_ambient[] = { 0.1f, 0.1f, 0.1f, 1.0f };
    GLfloat mat_diffuse[] = { 0.5f, 0.5f, 0.5f, 1.0f };
    GLfloat mat_specular[] = { 0.1f, 0.1f, 0.1f, 1.0f };
    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, mat_ambient);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_diffuse);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
    glMateriali(GL_FRONT_AND_BACK, GL_SHININESS, 40);
	
    GLfloat light_pos[] = {0.0f, 0.0f, 1.0f, 0.0f};
    glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
    glEnable(GL_LIGHT0);
	
    glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
    glEnable(GL_COLOR_MATERIAL);
    glColor3f(0.5f, 0.5f, 0.5f);
    glClearColor(0.3f, 0.3f, 1.0f, 0.0f);

	glViewport(rect.origin.x, rect.origin.y, rect.size.width, rect.size.height);
	glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);

    float aspect = rect.size.width/rect.size.height;
    camera.look_at(bounds, aspect);
    arcball.bounding_sphere(bounds.center, bounds.radius);
	
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    camera.apply();

    glMatrixMode(GL_MODELVIEW);
    arcball.apply_transform();
	
    if (will_light_scene) 
		glEnable(GL_LIGHTING); 
	else 
		glDisable(GL_LIGHTING);

	if (m) {
		
		glDisable(GL_POLYGON_OFFSET_LINE);

		//if (!will_draw_edges)
			mx_render_model(*m);
		
		if(will_draw_edges && eslim) {
			glEnable(GL_POLYGON_OFFSET_LINE);
			glPolygonOffset(-1.0, -1.0);
		}
		
		if( will_draw_edges && eslim )
		{
			glPushAttrib(GL_LIGHTING_BIT);
			glDisable(GL_LIGHTING);
			glC(0.25, 0.25, 0.25);
			
			glBegin(GL_LINES);
			for(uint i=0; i<eslim->edge_count(); i++)
			{
				const MxQSlimEdge *e = eslim->edge(i);
				glV(m->vertex(e->v1));
				glV(m->vertex(e->v2));
			}
			glEnd();
			
			glPopAttrib();
		}
		
		if( will_draw_quadrics )
		{
			glPushAttrib(GL_LIGHTING_BIT|GL_TRANSFORM_BIT);
			glDisable(GL_COLOR_MATERIAL);
			glEnable(GL_NORMALIZE);
			mx_quadric_shading(MX_GREEN_ELLIPSOIDS);
			for(VID v=0; v<m->vert_count(); v++)
				if( m->vertex_is_valid(v) )
					draw_quadric(v);
			
			glPopAttrib();
		}
		else if( 0 && selected_vertex != MXID_NIL &&
				m->vertex_is_valid(selected_vertex) )
		{
			glDisable(GL_COLOR_MATERIAL);
			mx_quadric_shading(MX_GREEN_ELLIPSOIDS);
			draw_quadric(selected_vertex);
		}
		
	}
	
	[[self openGLContext] flushBuffer];
    arcball.unapply_transform();
}

- (BOOL)application:(NSApplication *)theApplication 
		   openFile:(NSString *)filename
{
	open_file(filename);
	if (m) {
		[NSApp setTargetMax:m->face_count()];
		[NSApp updateStatus];
	}
	[self setNeedsDisplay:YES];
	return YES;
}

- (void)application:(NSApplication*)theApplication
		  openFiles:(NSArray*)files
{
	open_file([files objectAtIndex:0]);
	if (m) {
		[NSApp setTargetMax:m->face_count()];
		[NSApp updateStatus];
	}
	[self setNeedsDisplay:YES];
}

- (void)mouseDown:(NSEvent *)theEvent
{
	NSPoint p = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	if (NSPointInRect(p, [self frame])) {
		int xy[2];
		xy[0] = p.x;
		xy[1] = [self frame].size.height - p.y;
		switch ([theEvent type]) {
			case NSLeftMouseDown:
				arcball.mouse_down(xy, [theEvent modifierFlags]&NSControlKeyMask ? 3 : 1);
				break;
			case NSRightMouseDown:
				arcball.mouse_down(xy, 2);
				break;
			case NSOtherMouseDown:
				arcball.mouse_down(xy, 3);
				break;
		}	
		last[0] = xy[0];
		last[1] = xy[1];
		[self setNeedsDisplay:YES];
		isMovingMouse = true;
	}
}
- (void)rightMouseDown:(NSEvent *)theEvent
{
	[self mouseDown:theEvent];
}

- (void)rightMouseUp:(NSEvent *)theEvent
{
	[self mouseUp:theEvent];
}

- (void)rightMouseDragged:(NSEvent *)theEvent
{
	[self mouseDragged:theEvent];
}

- (void)mouseDragged:(NSEvent *)theEvent
{
	if (isMovingMouse) {		
		int xy[2];
		NSPoint p = [self convertPoint:[theEvent locationInWindow] fromView:nil];
		xy[0] = p.x;
		xy[1] = [self frame].size.height - p.y;
		switch ([theEvent type]) {
			case NSLeftMouseDragged:
				arcball.mouse_drag(xy, last, [theEvent modifierFlags]&NSControlKeyMask ? 3 : 1);
				break;
			case NSRightMouseDragged:
				arcball.mouse_drag(xy, last, 2);
				break;
			case NSOtherMouseDragged:
				arcball.mouse_drag(xy, last, 3);
				break;
		}	
		last[0] = xy[0];
		last[1] = xy[1];
		[self setNeedsDisplay:YES];
	}
}

- (void)mouseUp:(NSEvent *)theEvent
{
	if (isMovingMouse) {		
		int xy[2];
		NSPoint p = [self convertPoint:[theEvent locationInWindow] fromView:nil];
		xy[0] = p.x;
		xy[1] = [self frame].size.height - p.y;
		switch ([theEvent type]) {
			case NSLeftMouseUp:
				arcball.mouse_up(xy, [theEvent modifierFlags]&NSControlKeyMask ? 3 : 1);
				break;
			case NSRightMouseUp:
				arcball.mouse_up(xy, 2);
				break;
			case NSOtherMouseUp:
				arcball.mouse_up(xy, 3);
				break;
		}	
		[self setNeedsDisplay:YES];
		isMovingMouse = false;
	}
}

@end
